@@ -8,13 +8,13 @@ from lakshmi.table import Table
 
 
 class Account:
-    """Class representing an account."""
+    """Class representing an account (collection of assets)."""
 
     def __init__(self, name, account_type):
         """
-        Arguments:
+        Args:
           name: Printable name for this account.
-          account_type: Type of this account.
+          account_type: Type of this account (string).
         """
         self._name = name
         self.account_type = account_type
@@ -22,6 +22,7 @@ class Account:
         self._cash = 0
 
     def to_dict(self):
+        """Returns a dict representing this object."""
         d = {'Name': self._name,
              'Account Type': self.account_type}
         if self._assets:
@@ -32,6 +33,16 @@ class Account:
 
     @classmethod
     def from_dict(cls, d):
+        """Returns a new object specified by dictionary d.
+
+        This is reverse of to_dict.
+        Args:
+            d: A dictionary (usually the output of to_dict).
+
+        Returns: A new Account object.
+
+        Raises: AssertionError if d cannot be parsed correctly.
+        """
         ret_obj = Account(d.pop('Name'), d.pop('Account Type'))
         for asset_dict in d.pop('Assets', []):
             ret_obj.add_asset(from_dict(asset_dict))
@@ -40,6 +51,7 @@ class Account:
         return ret_obj
 
     def string(self):
+        """Returns a string representation of this object."""
         table = Table(2)
         table.add_row(['Name:', f'{self._name}'])
         table.add_row(['Type:', f'{self.account_type}'])
@@ -52,48 +64,86 @@ class Account:
         return table.string(tablefmt='plain')
 
     def add_asset(self, asset, replace=False):
+        """Add an asset to this Account.
+
+        Args:
+            asset: A lakshmi.assets.Asset object. The short_name of the
+            asset must be unique in this account (unless replace is set
+            to True).
+            replace: If true, this asset will replace any other asset
+            with the same short_name.
+
+        Returns: The new account.
+
+        Raises: AssertionError if another asset with the same short_name
+        is present in the account (unless replace is True).
+        """
         assert replace or asset.short_name() not in self._assets, (
             f'Attempting to add duplicate Asset: {asset.short_name()}')
         self._assets[asset.short_name()] = asset
         return self
 
     def assets(self):
+        """Retuns all assets in this account."""
         return self._assets.values()
 
     def set_assets(self, assets):
+        """Replaces all assets in this account with provided assets.
+
+        Args:
+            assets: A list of lakshmi.assets.Asset
+        """
         self._assets = {}
         for asset in assets:
             self.add_asset(asset)
 
     def get_asset(self, short_name):
+        """Returns an asset specified by short_name."""
         return self._assets[short_name]
 
     def remove_asset(self, short_name):
+        """Removes the asset in this account specified by short_name."""
         del self._assets[short_name]
 
     def name(self):
+        """Returns the name of this account."""
         return self._name
 
     def add_cash(self, delta):
+        """Adds cash to this account."""
         self._cash += delta
 
     def available_cash(self):
+        """Returns the available cash in this account."""
         return self._cash
 
 
 class AssetClass:
-    """(Tree of) Asset classes."""
+    """This class represents (a tree of) Asset classes.
+
+    Each asset class is specified by a name and can optionally have child
+    asset classes. The child asset classes have a ratio (float) associated
+    with them and the sum of all ratios must add to 1.0.
+
+    Example of asset classes: Stocks, Bonds, US, International, Small Cap,
+    Large Cap, etc.
+    Some asset classes can have children asset classes. E.g.
+    Stock -> US (60%) and International (40%).
+    """
 
     def __init__(self, name):
+        """Returns a new AssetClass object named name."""
         self.name = name
         self._children = []
         # Populated when _validate is called.
         self._leaves = None
 
     def children(self):
+        """Returns a list of all the children of this asset class."""
         return self._children
 
     def to_dict(self):
+        """Returns a dict representing this object."""
         d = {'Name': self.name}
         if self._children:
             d['Children'] = [{'Ratio': ratio} | child.to_dict()
@@ -102,6 +152,16 @@ class AssetClass:
 
     @classmethod
     def from_dict(cls, d):
+        """Returns a new object specified by dictionary d.
+
+        This is reverse of to_dict.
+        Args:
+            d: A dictionary (usually the output of to_dict).
+
+        Returns: A new AssetClass object.
+
+        Raises: AssertionError if d cannot be parsed correctly.
+        """
         ret_obj = AssetClass(d.pop('Name'))
         for child_dict in d.pop('Children', []):
             ret_obj.add_subclass(
@@ -118,14 +178,27 @@ class AssetClass:
         return ret_val
 
     def add_subclass(self, ratio, asset_class):
+        """Add a subclass (asset class) to this asset class.
+
+        Args:
+            ratio: The ratio of this asset class.
+            asset_class: An AssetClass object representing the child class.
+        """
         self._children.append((asset_class, ratio))
         # leaves is not upto date now, need validation again.
         self._leaves = None
         return self
 
     def _validate(self):
-        """Returns a tuple (leaf names, class names) for the subtree."""
-        # Check if all percentages add up to 100%
+        """Internal helper method to validate this asset class.
+
+        Returns: A tuple (leaf names, class names) for the subtree containing
+        set of all the leaf asset class names and list of all (including
+        non-leaf) asset classes.
+
+        Raises: AssertionError If ratio is not a valid float in [0, 1] or if
+        the sum of ratio of all the sub-classes doesn't add up to 1.0.
+        """
         if not self._children:
             self._leaves = {self.name}
             return self._leaves, [self.name]
@@ -141,12 +214,25 @@ class AssetClass:
             self._leaves.update(temp_leafs)
             class_names += temp_classes
 
+        # Check if all percentages add up to 100%
         assert abs(total - 1) < 1e-6, (
             f'Sum of sub-classes is not 100% (actual: {total * 100}%)')
 
         return self._leaves, class_names
 
     def validate(self):
+        """Validates if this asset class tree has any errors.
+
+        This function checks the ratios and also if the asset class tree has
+        any duplicate asset class names. It also sets an internal field which
+        is used by few other methods in this class. Those methods will raise
+        an exception if they are called without calling validate.
+
+        Returns: self
+
+        Raises: AssertionError if there are any duplicate asset classes in the
+        tree.
+        """
         unused_leaves, all_class_names = self._validate()
         duplicates = set([x for x in all_class_names
                           if all_class_names.count(x) > 1])
@@ -155,14 +241,33 @@ class AssetClass:
         return self
 
     def _check(self):
+        """Internal method to check if validate has been called."""
         assert self._leaves, (
             'Need to validate AssetAllocation before using it.')
 
     def find_asset_class(self, asset_class_name):
-        """Returns a tuple of object representing asset_class_name and its
-        desired ratio.
+        """Returns asset_class_name object and its desired ratio.
 
-        Returns None if asset_class_name is not found."""
+        validate() must be called before calling this method.
+        This methods searches for an asset class object with name
+        asset_class_name in the current asset tree and returns the found
+        object and the effective _absolute_ ratio of that asset class as
+        a tuple. If the asset_class_name is not found, this method returns
+        None. For example, if the asset class tree is:
+        All -> Stocks 60%, Bonds 40%
+        Stocks -> US 60%, Intl 40%
+        find_asset_class('US') will return asset class representing the 'US'
+        node and 0.36 as ratio (= 0.6 * 0.6).
+
+        Args:
+            asset_class_name: A string representing the name to be searched.
+
+        Returns: (Found asset class object, absolute ratio (float)) or None
+        if the asset_class_name is not found in this asset class tree.
+
+        Raises: AssertionError if validate is not callled before calling this
+        method.
+        """
         self._check()
 
         if self.name == asset_class_name:
@@ -176,24 +281,62 @@ class AssetClass:
         return None
 
     def leaves(self):
+        """Returns all leaf asset class names.
+
+        validate() must be called before calling this method.
+
+        Returns: A list of leaf asset class names.
+
+        Raises: AssertionError if validate is not callled before calling this
+        method.
+        """
         self._check()
         return self._leaves
 
     def value_mapped(self, money_allocation):
-        """Returns how much money is mapped to this Asset Class or its
-        children.
+        """Returns how much money is mapped to this Asset Class.
 
-        Arguments:
-          money_allocation: A map of leaf_class_name -> money.
+        validate() must be called before calling this method.
+        Given a money allocation, this class return the amount of money mapped
+        to this asset class or its childen.
+
+        Ars:
+          money_allocation: A map of leaf_class_names (string) -> money
+          (float).
+
+        Returns: Total amount of money mapped to this asset class.
+
+        Raises: AssertionError if validate is not callled before calling this
+        method.
         """
         self._check()
         return sum([value for name, value in money_allocation.items()
                     if name in self._leaves])
 
     class Allocation:
+        """This class is a convenience class to represent the return value of
+        return_allocation method. This class represents a partcular node of
+        AssetClass + its direct children and also the money allocated to the
+        asset class and its direct children. It is meant to be used as a
+        data-only class.
+        """
         class Children:
+            """Class representing a child of Allocation class. This class is
+            meant to be used as a data-only class.
+            """
             def __init__(self, name, actual_allocation, desired_allocation,
                          value, value_difference):
+                """
+                Args:
+                    name: Name of the child asset class.
+                    actual_allocation: Actual allocation (ratio, float).
+                    desired_allocation: Desired asset allocation (ratio,
+                    float).
+                    value: The absolute amount of money allocated to this
+                    child.
+                    value_difference: The difference between desired and
+                    actual allocation.
+                """
                 self.name = name
                 self.actual_allocation = actual_allocation
                 self.desired_allocation = desired_allocation
@@ -201,11 +344,24 @@ class AssetClass:
                 self.value_difference = value_difference
 
         def __init__(self, name, value):
+            """
+            Args:
+                name: Name of this asset class.
+                value: The absolute amount of money allocated to this asset
+                class.
+            """
             self.name = name
             self.value = value
             self.children = []
 
         def add_child(self, name, actual, desired):
+            """Add a child asset to this asset class.
+
+            Args:
+                name: Name of the child asset class.
+                actual: The actual amount of money allocated to this child.
+                desired: The desired alloction of money for this child.
+            """
             self.children.append(
                 self.Children(name, actual, desired, actual * self.value,
                               (desired - actual) * self.value))
@@ -213,12 +369,11 @@ class AssetClass:
     def return_allocation(self, money_allocation, levels=-1):
         """Returns actual and desired allocation based on how money is allocated.
 
-        Arguments:
+        Args:
           money_allocation: A map of leaf_class_name -> money.
           levels: How many levels of child allocation to return (-1 = all).
 
-        Returns:
-        A list of ActualAllocation objects (for itself and any child classes
+        Returns: A list of Allocation objects (for itself and any child classes
         based on the levels flag).
         """
         value = self.value_mapped(money_allocation)
@@ -249,22 +404,36 @@ class AssetClass:
 
 
 class Portfolio:
+    """Top-level class representing a portfolio.
+
+    A portfolio consists of multiple accounts (Account objects). This
+    class provides helper methods to add/delete accounts etc. This class
+    also provides methods for portfolio level operations.
+    """
     def __init__(self, asset_classes):
+        """
+        Args:
+            asset_classes: An AssetClass object representing the desired asset
+            allocation.
+        """
         self._accounts = {}
         self.asset_classes = asset_classes.validate()
         self._leaf_asset_classes = asset_classes.leaves()
 
     def save(self, filename):
+        """Save this portfolio to a file."""
         with open(filename, 'w') as f:
             yaml.dump(self.to_dict(), f, sort_keys=False)
 
     @classmethod
     def load(cls, filename):
+        """Loads and returns portfolio from file."""
         with open(filename) as f:
             d = yaml.load(f.read(), Loader=yaml.SafeLoader)
         return Portfolio.from_dict(d)
 
     def to_dict(self):
+        """Returns a dictionary representation of this portfolio."""
         d = {'Asset Classes': self.asset_classes.to_dict()}
         if self._accounts:
             d['Accounts'] = [account.to_dict()
@@ -273,6 +442,7 @@ class Portfolio:
 
     @classmethod
     def from_dict(cls, d):
+        """Returns a portfolio represented by dict d (reverse of to_dict)."""
         ret_obj = Portfolio(AssetClass.from_dict(d.pop('Asset Classes')))
         for account_dict in d.pop('Accounts', []):
             ret_obj.add_account(Account.from_dict(account_dict))
@@ -280,6 +450,20 @@ class Portfolio:
         return ret_obj
 
     def add_account(self, account, replace=False):
+        """Add an account to this portfolio.
+
+        This method adds a new account to this portfolio. The account
+        name must be unique unless replace is set to True.
+
+        Returns: self object with account added.
+
+        Raises:
+            AssertionError: If any of the assets are mapped to asset classes
+            that doesn't match the asset classes given the asset allocation
+            provided when creating this class.
+            AssertError: If another account with the same exists in the
+            portfolio and replace=False.
+        """
         for asset in account.assets():
             for asset_class in asset.class2ratio.keys():
                 assert asset_class in self._leaf_asset_classes, (
@@ -292,19 +476,25 @@ class Portfolio:
         return self
 
     def remove_account(self, account_name):
+        """Delete account specifed by account_name."""
         del self._accounts[account_name]
 
     def accounts(self):
+        """Returns a list (dict_values) of accounts."""
         return self._accounts.values()
 
     def get_account(self, name):
+        """Return Account object represented by name."""
         return self._accounts[name]
 
     def get_account_name_by_substr(self, account_str):
         """Returns account name whose name partially matches account_str.
 
-        This method throws an AssertionError if more than one or none of the
-        accounts match the account_str.
+        Args:
+            account_str: String used for sub-string matching (case-sensitive).
+
+        Raises: AssertionError if more than one or none of the accounts match
+        the account_str.
         """
 
         matched_accounts = list(
@@ -319,10 +509,19 @@ class Portfolio:
         return matched_accounts[0]
 
     def get_asset_name_by_substr(self, account_str='', asset_str=''):
-        """Returns a tuple account_name, asset_name where account name
+        """Returns asset names given a sub-string for account and asset.
+
+        Args:
+            account_str: String used for sub-string matching with account
+            names (case-sensitive).
+            asset_str: String used for sub-string matching with asset
+            names or asset short names (case-sensitive).
+
+        Returns: A tuple account_name, asset_name where account name
         partially matches account_str and asset name partially matches
         asset_str or asset short name == asset_str
-        Raises AssertionError if none or more than one asset matches the
+
+        Raises: AssertionError if none or more than one asset matches the
         sub-strings."""
         matched_assets = list(filter(
             lambda x: x[0].count(account_str) and (
@@ -341,8 +540,17 @@ class Portfolio:
         return matched_assets[0][0], matched_assets[0][2]
 
     def what_if(self, account_name, asset_name, delta):
-        """Runs a whatif scenario if asset_name in account_name is changed by
-        delta."""
+        """Changes value of an asset by delta.
+
+        This method runs a whatif scenario if asset_name in account_name
+        is changed by delta. The account cash balance is changed with
+        -delta to balance the transaction.
+
+        Args:
+            account_name: String representing the account name.
+            asset_name: String representing the asset name.
+            delta: A positive or negative delta for money.
+        """
         account = self.get_account(account_name)
         asset = account.get_asset(asset_name)
         asset.what_if(delta)
@@ -350,9 +558,17 @@ class Portfolio:
         account.add_cash(-delta)
 
     def what_if_add_cash(self, account_name, cash_delta):
+        """Changes available cash balance of an account by delta."""
         self.get_account(account_name).add_cash(cash_delta)
 
     def get_what_ifs(self):
+        """Returns all what_ifs set by previous methods.
+
+        Returns: A tuple of two table.Table representing
+        account whatifs and asset what ifs respectively. The first
+        table has columns Account and Cash; and the second table has
+        columns Account, Asset and Delta (representing money).
+        """
         account_whatifs = Table(
             2,
             headers=['Account', 'Cash'],
@@ -375,13 +591,14 @@ class Portfolio:
         return account_whatifs, asset_whatifs
 
     def reset_what_ifs(self):
+        """Reset all previously set what ifs in this portfolio."""
         for account in self.accounts():
             account.add_cash(-account.available_cash())
             for asset in account.assets():
                 asset.what_if(-asset.get_what_if())
 
     def total_value(self):
-        """Returns total of all assets added."""
+        """Returns total value of the portfolio."""
         total = 0.0
         for account in self.accounts():
             total += account.available_cash()
@@ -391,7 +608,17 @@ class Portfolio:
 
     # TODO: Renmame this to list_assets for consistency.
     def assets(self, short_name=False, quantity=False):
-        """Returns all the assets."""
+        """Returns all the assets.
+
+        Args:
+            short_name: If set, returns the short name of the asset as well.
+            quantity: If set, returns the shares/quantity of the asset for
+            the assets that support it.
+
+        Returns: A table.Table object representing all the assets.
+        The columns correspond to Account name, short name (optional),
+        quantity (optional), asset name and value.
+        """
         table = Table(
             3 + short_name + quantity,
             headers=(['Account']
@@ -414,7 +641,13 @@ class Portfolio:
         return table
 
     def list_lots(self):
-        """Returns all the lots in the portfolio as table.Table."""
+        """Returns all the tax lots in the portfolio.
+
+        Returns: A table.Table object representing tax lots
+        for assets that support it. The columns of the returned table are
+        short name (of asset), date of lot, cost basis of lot, gain (+ve or
+        -ve) and percentage gain.
+        """
         table = Table(
             5,
             headers=['Short Name', 'Date', 'Cost', 'Gain', 'Gain%'],
@@ -431,8 +664,12 @@ class Portfolio:
         return table
 
     def asset_location(self):
-        """Returns asset location as a list of [asset class, account_type,
-        percentage, value]."""
+        """Returns asset location of this portfolio.
+
+        Returns: A table.Table object representing the asset location.
+        The columns of the table correspond to Asset class name, account type,
+        percentage allocation to account type and total monetary value.
+        """
         class2type = {}  # Mapping of asset class -> account type -> money
         for account in self.accounts():
             for asset in account.assets():
@@ -463,6 +700,7 @@ class Portfolio:
         return table
 
     def _get_asset_class_to_value(self):
+        """Returns asset class name -> money allocated to it."""
         asset_class_to_value = {}
 
         for account in self.accounts():
@@ -474,6 +712,20 @@ class Portfolio:
         return asset_class_to_value
 
     def asset_allocation_tree(self, levels=-1):
+        """Returns asset allocation in long vertical tree format.
+
+        Args:
+            levels: The max depth of asset allocation tree (-1 = all levels).
+
+        Returns: table.Table object representing the asset allocation. The
+        table has multiple sections separated by empty row, each corresponding
+        to an asset class and it's children asset class. The first row in a
+        section is the asset class name followed by ":". The remaining rows
+        correspond to the direct children of the asset class. For these
+        rows the columns correspond to asset class name, actual percentage of
+        money allocated to it, the desired percentage of money allocated, and
+        the actual money allocated to it.
+        """
         table = Table(4,
                       headers=['Class', 'Actual%', 'Desired%', 'Value'],
                       coltypes=['str', 'percentage', 'percentage', 'dollars'])
@@ -495,6 +747,29 @@ class Portfolio:
         return table
 
     def asset_allocation(self, asset_class_list):
+        """Returns asset allocation across the asset classes provided.
+
+        This method returns the asset allocation, but only across the asset
+        classes provided via asset_class_list. This requires that the asset
+        class list provided represents a "cut" of the asset class tree. What
+        it means is that the asset list should cover the asset tree completely.
+        None of the asset classes should be a descendent or ancestor of another
+        and the sum of actual allocation across the asset class list must add
+        up to 100%.
+
+        Args:
+            asset_class_list: A list of strings representing the asset classes
+            across which the asset allocation should be returned.
+
+        Returns: A table.Table object representing the asset allocation. The
+        columns represent asset class name, actual percentage allocation to
+        the asset class, the desired percentage allocation of money to the
+        asset class, the absolute money allocated to it and difference from
+        the desired amount of money allocated.
+
+        Raises: AssertionError is the asset_class_list is not a proper
+        "cut" of the asset class tree.
+        """
         flat_asset_class = AssetClass('Root')
         for asset_class in asset_class_list:
             found = self.asset_classes.find_asset_class(asset_class)
@@ -523,7 +798,23 @@ class Portfolio:
         return table
 
     def asset_allocation_compact(self):
-        """Prints a 'compact' version of AA."""
+        """Returns asset allocation in long horizontal format.
+
+        This method is similar to asset_allocaton_tree method in terms of
+        informaiton returned, but this method returns the asset allocation
+        in horizontal format.
+
+        Returns: A table.Table object representing the portfolio's asset
+        allocation. Each row corresponds to a path in the asset allocation
+        tree. The columns are grouped into groups of three for each node
+        in the tree (except the final two columns). The three columns for
+        each node correspond to the asset class name, actual percentage
+        of assets mapped to the asset class, and the desired percentage
+        of assets. The final two columns are corresponding to the leaf
+        nodes of asset allocation: They contain the actual value of assets
+        mapped to the leaf node, and the difference from the desired
+        value of assets mapped to the leaf node, respectively.
+        """
         def find_index(class_name, ret_list):
             # We assume ret_list has atleast one entry.
             names_list = [row[-3] for row in ret_list]
